import cv2
import os
import copy


def read_input(path):
    img = cv2.imread(path)
    img = img[:,:1024,0]
    print('img shape: ',img.shape)
    return img


def denoised(img, output_path):
    denoised_img = cv2.fastNlMeansDenoising(img,None,30,7,30)
    cv2.imwrite(os.path.join(output_path, 'denoised.jpg'),denoised_img)
    return denoised_img
    
    
def binarized(img, output_path, threshold = -1):
    th, img_th = cv2.threshold(img, 0, 255, cv2.THRESH_BINARY+cv2.THRESH_OTSU)
    print(' OT threshold: ',th)
    #print('threshold using: ',th+10)
    th, img_th = cv2.threshold(img, th+10, 255 , cv2.THRESH_BINARY)
    cv2.imwrite(os.path.join(output_path,'binarized.jpg'),img_th)
    #print('threshold using: ',th)
    return img_th, th

def draw_rectangle(bbox, image, wh=False, color=(0,0,0), thickness=1):
    start_point = (bbox[0],bbox[1])
    end_point = (bbox[2],bbox[3])
    #print(start_point)
    if wh:
        end_point = (bbox[0]+bbox[2], bbox[1]+bbox[3])
    #print(end_point)
    image = cv2.rectangle(image, start_point, end_point, color, thickness)
    return image

def draw_bboxes(image, bboxes,wh=False):
    image = copy.deepcopy(image)
    for _,bbox in enumerate(bboxes):
        if wh:          #bbox in x,y,w,h format
            bbox = [bbox[0], bbox[1], bbox[0]+bbox[2], bbox[1]+bbox[3]] #x1,y1,x2,y2
        image = draw_rectangle(bbox, image,thickness=2)
    return image


def draw_sorted_boxes(image, row_bboxes):
    for i,row_box_list in enumerate(row_bboxes):
        img = copy.deepcopy(image)
        for _,bbox in enumerate(row_box_list):
            img = draw_rectangle(bbox, img,thickness=2)
        cv2.imwrite(os.path.join('dummy','boxed_'+str(i)+'.jpg'),img)


def generate_row_images(image, lines, bboxes_row_sorted, output_path):
    #lines = lines[1:len(lines)-1]
    omit = 0
    output = []
    cell_rows = []
    r_bboxes_local = []

    output_row_image_path = os.path.join(output_path,"rows")
    if not os.path.exists(output_row_image_path):
        os.makedirs(output_row_image_path)

    if len(lines)%2==0:
        omit = 2
    else:
        omit = 1

    gate_row_count = 0

    for i in range(0, len(lines)-omit):   #some portion from bottom are omitted    
        bboxes = bboxes_row_sorted[i]
        curr_line = lines[i]
        next_line = lines[i+1]
        x1, y1 = curr_line[0], curr_line[1]
        x2, y2 = next_line[2], next_line[3] 
        row_image = image[y1:y2+1, x1:x2+1]

        bboxes_local = [[bbox[0], bbox[1]-y1, bbox[2], bbox[3]-y1] for bbox in bboxes]

        if i%2 == 0:
            next_next_line = lines[i+2]
            x2, y2 = next_next_line[2], next_next_line[3]
            cell_row = image[y1:y2+1, x1:x2+1]
            #clean_rows(gate_row, gate_row_count, components, bboxes_row_sorted[i+1], next_line[1]-curr_line[1],output_path)
            gate_row_count += 1
            cell_rows.append(cell_row)

        output.append(row_image)
        r_bboxes_local.append(bboxes_local)
        # to check out correctedness
        row_image = draw_bboxes(row_image, bboxes_local)
        cv2.imwrite(os.path.join(output_row_image_path,'row_'+str(i)+'.png'),row_image)

    return output, r_bboxes_local, cell_rows


def calculate_distance(comps):
    distance_x = []
    for i in range(1,len(comps)):
        pre_bbox = comps[i-1]
        bbox = comps[i]
        distance_x.append(bbox[0]-pre_bbox[2]) #next_box_x1 - prev_box_x2
    #print('distance x: ',distance_x)
    return distance_x

def merging(merge):
    return [min(x[0] for x in merge),min(x[1] for x in merge),max(x[2] for x in merge),max(x[3] for x in merge)]

def splating_bbox(components, img):
    height, width = img.shape
    c = []
    for comp in components:
        comp = [max(0,comp[0]-3), max(0,comp[1]-3), min(width-1,comp[2]+3), min(height-1,comp[3]+3)]
        c.append(comp)
    return c

def cropped_coordinates(height,width,component):
    return component[0], 0, component[2], height-1

def save_cell_image(image, row_no, comp_no, path):
    output_image_path = os.path.join(path,"cell_images",str(row_no))
    if not os.path.exists(output_image_path):
        os.makedirs(output_image_path)
    cv2.imwrite(os.path.join(output_image_path,str(comp_no)+'.jpg'),image)